#!/usr/bin/env node

/**
 * Enhanced AI-to-AI Loop System Test
 * Tests the new acknowledgment processing, infinite loop mechanism, and sequential processing
 */

const axios = require('axios');

class EnhancedAIToAILoopTester {
    constructor() {
        this.mcpServerUrl = 'http://localhost:7878';
        this.testResults = [];
        this.currentLoopId = null;
        this.testStartTime = Date.now();
    }

    async runAllTests() {
        console.log('🚀 Starting Enhanced AI-to-AI Loop System Tests...\n');

        try {
            // Test 1: Verify MCP server is running
            await this.testMCPServerHealth();

            // Test 2: Test new acknowledgment function mapping
            await this.testAcknowledgmentFunctionMapping();

            // Test 3: Start AI-to-AI loop
            await this.testStartAIToAILoop();

            // Test 4: Test acknowledgment processing and next iteration trigger
            await this.testAcknowledgmentProcessing();

            // Test 5: Test infinite loop continuation
            await this.testInfiniteLoopContinuation();

            // Test 6: Test error recovery mechanisms
            await this.testErrorRecovery();

            // Test 7: Test sequential processing
            await this.testSequentialProcessing();

            // Test 8: Test loop stopping with stploop command
            await this.testLoopStopping();

            // Display results
            this.displayTestResults();

        } catch (error) {
            console.error(`❌ Test suite failed: ${error.message}`);
            process.exit(1);
        }
    }

    async testMCPServerHealth() {
        console.log('🔍 Test 1: MCP Server Health Check');
        
        try {
            const response = await axios.get(`${this.mcpServerUrl}/health`, { timeout: 5000 });
            
            if (response.status === 200) {
                this.addTestResult('MCP Server Health', true, 'Server is running and responsive');
            } else {
                this.addTestResult('MCP Server Health', false, `Unexpected status: ${response.status}`);
            }
        } catch (error) {
            this.addTestResult('MCP Server Health', false, `Server not accessible: ${error.message}`);
            throw error;
        }
    }

    async testAcknowledgmentFunctionMapping() {
        console.log('🔍 Test 2: Acknowledgment Function Mapping');
        
        try {
            // Test acknowledge_agent_response_Zai function
            const response = await this.sendMCPRequest('acknowledge_agent_response_Zai', {
                loopId: 'test_loop_123',
                agentResponse: 'Test acknowledgment for function mapping verification'
            });

            if (response.error) {
                if (response.error.message.includes('No pending response found')) {
                    this.addTestResult('Zai Acknowledgment Function', true, 'Function is properly mapped (no pending response is expected)');
                } else {
                    this.addTestResult('Zai Acknowledgment Function', false, `Unexpected error: ${response.error.message}`);
                }
            } else {
                this.addTestResult('Zai Acknowledgment Function', true, 'Function executed successfully');
            }
        } catch (error) {
            this.addTestResult('Zai Acknowledgment Function', false, `Function mapping failed: ${error.message}`);
        }
    }

    async testStartAIToAILoop() {
        console.log('🔍 Test 3: Start AI-to-AI Loop');
        
        try {
            const response = await this.sendMCPRequest('activate_infinite_loop', {
                message: 'actloop test enhanced ai to ai functionality',
                aiToAi: true,
                maxIterations: 999999,
                interval: 3000
            });

            if (response.error) {
                this.addTestResult('Start AI-to-AI Loop', false, `Failed to start loop: ${response.error.message}`);
                return;
            }

            const content = response.content?.[0]?.text || '';
            if (content.includes('AI-to-AI infinite loop activated')) {
                // Extract loop ID from response
                const loopMatch = content.match(/ai2ai_\d+_\d+/);
                if (loopMatch) {
                    this.currentLoopId = loopMatch[0];
                    this.addTestResult('Start AI-to-AI Loop', true, `Loop started successfully: ${this.currentLoopId}`);
                } else {
                    this.addTestResult('Start AI-to-AI Loop', false, 'Loop started but could not extract loop ID');
                }
            } else {
                this.addTestResult('Start AI-to-AI Loop', false, 'Unexpected response content');
            }
        } catch (error) {
            this.addTestResult('Start AI-to-AI Loop', false, `Error starting loop: ${error.message}`);
        }
    }

    async testAcknowledgmentProcessing() {
        console.log('🔍 Test 4: Acknowledgment Processing and Next Iteration Trigger');
        
        if (!this.currentLoopId) {
            this.addTestResult('Acknowledgment Processing', false, 'No active loop ID available');
            return;
        }

        try {
            // Wait a moment for the first iteration to be generated
            await this.sleep(2000);

            // Send acknowledgment to trigger next iteration
            const response = await this.sendMCPRequest('acknowledge_agent_response_Zai', {
                loopId: this.currentLoopId,
                agentResponse: 'Test acknowledgment: I have implemented the requested improvements for enhanced AI-to-AI functionality. The system is now more robust with better error handling and infinite loop capabilities. Status: continue'
            });

            if (response.error) {
                this.addTestResult('Acknowledgment Processing', false, `Acknowledgment failed: ${response.error.message}`);
                return;
            }

            const content = response.content?.[0]?.text || '';
            if (content.includes('AI-to-AI loop continuing')) {
                this.addTestResult('Acknowledgment Processing', true, 'Acknowledgment processed and next iteration triggered');
            } else {
                this.addTestResult('Acknowledgment Processing', false, 'Acknowledgment processed but unexpected response');
            }
        } catch (error) {
            this.addTestResult('Acknowledgment Processing', false, `Error in acknowledgment processing: ${error.message}`);
        }
    }

    async testInfiniteLoopContinuation() {
        console.log('🔍 Test 5: Infinite Loop Continuation');
        
        if (!this.currentLoopId) {
            this.addTestResult('Infinite Loop Continuation', false, 'No active loop ID available');
            return;
        }

        try {
            // Check loop status multiple times to verify it continues
            let continuationCount = 0;
            const maxChecks = 3;

            for (let i = 0; i < maxChecks; i++) {
                await this.sleep(3000); // Wait for next iteration

                const statusResponse = await this.sendMCPRequest('get_loop_status', {
                    loopId: this.currentLoopId
                });

                if (!statusResponse.error) {
                    const content = statusResponse.content?.[0]?.text || '';
                    if (content.includes('isActive: true')) {
                        continuationCount++;
                        
                        // Send another acknowledgment to continue the loop
                        await this.sendMCPRequest('acknowledge_agent_response_Zai', {
                            loopId: this.currentLoopId,
                            agentResponse: `Iteration ${i + 2} completed: Enhanced AI-to-AI system is working correctly. Continuing infinite loop. Status: continue`
                        });
                    }
                }
            }

            if (continuationCount >= 2) {
                this.addTestResult('Infinite Loop Continuation', true, `Loop continued for ${continuationCount} iterations`);
            } else {
                this.addTestResult('Infinite Loop Continuation', false, `Loop only continued for ${continuationCount} iterations`);
            }
        } catch (error) {
            this.addTestResult('Infinite Loop Continuation', false, `Error testing loop continuation: ${error.message}`);
        }
    }

    async testErrorRecovery() {
        console.log('🔍 Test 6: Error Recovery Mechanisms');
        
        if (!this.currentLoopId) {
            this.addTestResult('Error Recovery', false, 'No active loop ID available');
            return;
        }

        try {
            // Test force next iteration (recovery mechanism)
            const response = await this.sendMCPRequest('clear_agent_busy_state', {});
            
            if (!response.error) {
                this.addTestResult('Error Recovery', true, 'Agent busy state cleared successfully');
            } else {
                this.addTestResult('Error Recovery', false, `Error recovery failed: ${response.error.message}`);
            }
        } catch (error) {
            this.addTestResult('Error Recovery', false, `Error testing recovery: ${error.message}`);
        }
    }

    async testSequentialProcessing() {
        console.log('🔍 Test 7: Sequential Processing');
        
        if (!this.currentLoopId) {
            this.addTestResult('Sequential Processing', false, 'No active loop ID available');
            return;
        }

        try {
            // Send multiple acknowledgments rapidly to test sequential processing
            const responses = await Promise.allSettled([
                this.sendMCPRequest('acknowledge_agent_response_Zai', {
                    loopId: this.currentLoopId,
                    agentResponse: 'Sequential test 1: Processing first acknowledgment'
                }),
                this.sendMCPRequest('acknowledge_agent_response_Zai', {
                    loopId: this.currentLoopId,
                    agentResponse: 'Sequential test 2: Processing second acknowledgment'
                })
            ]);

            const successCount = responses.filter(r => r.status === 'fulfilled' && !r.value.error).length;
            
            if (successCount >= 1) {
                this.addTestResult('Sequential Processing', true, `${successCount}/2 sequential acknowledgments processed`);
            } else {
                this.addTestResult('Sequential Processing', false, 'Sequential processing failed');
            }
        } catch (error) {
            this.addTestResult('Sequential Processing', false, `Error testing sequential processing: ${error.message}`);
        }
    }

    async testLoopStopping() {
        console.log('🔍 Test 8: Loop Stopping with stploop Command');
        
        if (!this.currentLoopId) {
            this.addTestResult('Loop Stopping', false, 'No active loop ID available');
            return;
        }

        try {
            const response = await this.sendMCPRequest('stop_ai_loops', {
                message: 'stploop'
            });

            if (response.error) {
                this.addTestResult('Loop Stopping', false, `Failed to stop loop: ${response.error.message}`);
                return;
            }

            const content = response.content?.[0]?.text || '';
            if (content.includes('stopped') || content.includes('terminated')) {
                this.addTestResult('Loop Stopping', true, 'Loop stopped successfully with stploop command');
            } else {
                this.addTestResult('Loop Stopping', false, 'Unexpected response to stop command');
            }
        } catch (error) {
            this.addTestResult('Loop Stopping', false, `Error stopping loop: ${error.message}`);
        }
    }

    async sendMCPRequest(toolName, args) {
        const request = {
            jsonrpc: "2.0",
            id: Date.now(),
            method: "tools/call",
            params: {
                name: toolName,
                arguments: args
            }
        };

        const response = await axios.post(`${this.mcpServerUrl}/mcp`, request, {
            headers: { 'Content-Type': 'application/json' },
            timeout: 10000
        });

        return response.data.result || response.data;
    }

    addTestResult(testName, passed, details) {
        this.testResults.push({
            name: testName,
            passed,
            details,
            timestamp: new Date().toISOString()
        });

        const status = passed ? '✅ PASS' : '❌ FAIL';
        console.log(`   ${status}: ${testName} - ${details}\n`);
    }

    displayTestResults() {
        const totalTests = this.testResults.length;
        const passedTests = this.testResults.filter(r => r.passed).length;
        const failedTests = totalTests - passedTests;
        const testDuration = Math.round((Date.now() - this.testStartTime) / 1000);

        console.log('=' .repeat(80));
        console.log('🧪 ENHANCED AI-TO-AI LOOP SYSTEM TEST RESULTS');
        console.log('=' .repeat(80));
        console.log(`📊 Total Tests: ${totalTests}`);
        console.log(`✅ Passed: ${passedTests}`);
        console.log(`❌ Failed: ${failedTests}`);
        console.log(`⏱️  Duration: ${testDuration}s`);
        console.log(`📈 Success Rate: ${Math.round((passedTests / totalTests) * 100)}%`);
        console.log('=' .repeat(80));

        if (failedTests > 0) {
            console.log('\n❌ FAILED TESTS:');
            this.testResults.filter(r => !r.passed).forEach(test => {
                console.log(`   • ${test.name}: ${test.details}`);
            });
        }

        if (passedTests === totalTests) {
            console.log('\n🎉 ALL TESTS PASSED! Enhanced AI-to-AI loop system is working correctly.');
        } else {
            console.log('\n⚠️  Some tests failed. Please review the implementation.');
        }
    }

    sleep(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }
}

// Run the tests
if (require.main === module) {
    const tester = new EnhancedAIToAILoopTester();
    tester.runAllTests().catch(error => {
        console.error('Test suite crashed:', error);
        process.exit(1);
    });
}

module.exports = EnhancedAIToAILoopTester;
